import requests
import yaml
import psycopg2
from pandas import json_normalize
from datetime import datetime
from Funcoes.consultaTabelas import tabela_existe
from Funcoes.criaTabelas import tabela_nova
from Config.pasta_local import pasta_local

# Coleta data e hora para inserir no banco
data_hora = datetime.now()

# Abre o arquivo YAML com as configurações necessários. Validar o caminho que está salvo os arquivos para execução.
nome_arquivo_yaml = pasta_local+'\Config\config.yaml'

if __name__ == '__main__':
    with open(nome_arquivo_yaml,encoding='utf-8') as file:
        config = yaml.safe_load(file)

# Coleta os dados do arquivo YAML
token = config['token']
db_name = config['connector-oltp']['db_name']
db_user = config['connector-oltp']['db_user']
db_password = config['connector-oltp']['db_password']
db_host = config['connector-oltp']['db_host']
db_schema = config['db_schema']

# Informa a tabela que trabalharemos nesta etapa do ETL
tabela = 'clickup_user'

# Comando SQL para criar a tabela
query = f'''
        CREATE TABLE {db_schema}.{tabela} (
            usuario_id VARCHAR(10),
            usuario_nome VARCHAR(100),
            usuario_email VARCHAR(100),
            data_hora TIMESTAMP
        );
        '''

# Verifica se já existe a tabela, caso não exista ela é criada
if tabela_existe(tabela,db_name,db_user,db_password,db_host):
    #   print(f'Tabela "{tabela}" já existe!')
      pass
else :
      tabela_nova(query,db_name,db_user,db_password,db_host)
     
# url de consulta do usuario
url = "https://api.clickup.com/api/v2/user"

# estrutura os headers conforme documentação utilizando o token salvo na pasta Config - arquivo config.yaml
headers = {'Authorization': f'{token}'}

# executa o request na url informando os headers
response = requests.get(url, headers=headers)

# coleta dos dados da respostsa
data = response.json()

# define o retorno da consulta em JSON
df = json_normalize(data)

# seleciona as 3 primeiras colunas
df = df.iloc[:, [0, 1, 2]]

# renomeia as colunas
df.rename(columns={'user.username': 'usuario_nome', 'user.id' : 'usuario_id', 'user.email' : 'usuario_email'}, inplace=True)

# reseta o index do df
df = df.reset_index(drop=True)

# insere a data e hora na tabela para controle de atualizações
df['data_hora'] = data_hora

# conecta no banco
try:
    connection = psycopg2.connect(
        user= db_user,
        password= db_password,
        host= db_host,
        port= "5432",
        database= db_name
    )
    cursor = connection.cursor()
    # print("Conexão realizada com sucesso!")

    # Testando a conexão
    cursor.execute("SELECT version();")
    record = cursor.fetchone()
    # print("Você está conectado ao - ", record, "\n")

except Exception as error:
    pass
    # print("Erro ao conectar ao banco de dados PostgreSQL", error)

# Executando a consulta TRUNCATE TABLE
truncate_query = f'TRUNCATE TABLE {db_schema}.{tabela};'
cursor.execute(truncate_query)

# Confirmando as alterações
connection.commit()

# Preparando a consulta SQL para inserção
insert_query = f'INSERT INTO {db_schema}.{tabela}  (usuario_id, usuario_nome, usuario_email, data_hora) ' + 'VALUES (%s, %s, %s, %s);'

# Iterando sobre as linhas do DataFrame e inserindo no banco
for index, row in df.iterrows():
    cursor.execute(insert_query, (row['usuario_id'], row['usuario_nome'], row['usuario_email'], row['data_hora']))

# Confirmando as alterações
connection.commit()

# Fechando o cursor atual
cursor.close()

# Fechando a conexão
if connection:
            cursor.close()
            connection.close()
