import requests
import yaml
import psycopg2
import pandas as pd
from pandas import json_normalize
from datetime import datetime
from Funcoes.consultaTabelas import tabela_existe
from Funcoes.criaTabelas import tabela_nova
from Config.pasta_local import pasta_local

# Coleta data e hora para inserir no banco
data_hora = datetime.now()

# Abre o arquivo YAML com as configurações necessários. Validar o caminho que está salvo os arquivos para execução.
nome_arquivo_yaml = pasta_local+'\Config\config.yaml'

if __name__ == '__main__':
    with open(nome_arquivo_yaml,encoding='utf-8') as file:
        config = yaml.safe_load(file)

# Coleta os dados do arquivo YAML
token = config['token']
db_name = config['connector-oltp']['db_name']
db_user = config['connector-oltp']['db_user']
db_password = config['connector-oltp']['db_password']
db_host = config['connector-oltp']['db_host']
db_schema = config['db_schema']

# Informa a tabela que trabalharemos nesta etapa do ETL
tabela = 'clickup_team'

# Comando SQL para criar a tabela
query = f'''
        CREATE TABLE {db_schema}.{tabela} (
            time_id VARCHAR(10),
            time_nome VARCHAR(50),
            usuario_id VARCHAR(10),
            usuario_nome VARCHAR(100),
            usuario_email VARCHAR(100),
            data_hora TIMESTAMP
        );
        '''

# Verifica se já existe a tabela, caso não exista ela é criada
if tabela_existe(tabela,db_name,db_user,db_password,db_host):
    #   print(f'Tabela "{tabela}" já existe!')
      pass
else :
      tabela_nova(query,db_name,db_user,db_password,db_host)

# url de consulta dos times
url = "https://api.clickup.com/api/v2/team"

# estrutura os headers conforme documentação utilizando o token salvo na pasta Config - arquivo config.yaml
headers = {'Authorization': f'{token}'}

# executa o request na url informando os headers
response = requests.get(url, headers=headers)

# coleta dos dados da respostsa
data = response.json()

# seleciona todos os times configurados na plataforma
times_plataforma = data['teams']

# verfica quantidade de times para realizar o looping com while
quantidade_times = len(times_plataforma)

# controle de times para sair do looping com while
controle_times = 0

# lopping com while para criar df dos times
while controle_times < quantidade_times:

# loop dos times na plataforma, desta maneira criamos o df com a descrição e nome dos times
    for i in times_plataforma:
        team_id = i['id']
        team_name = i['name']
        team_members = i['members']
    df = json_normalize(team_members)
    df['team.id'] = team_id
    df['team.name'] = team_name
# controle para saber se está na primeira "página".
    if controle_times == 0:
        df_final = df
# caso não esteja na primeira página do while concatena os dois dfs.
    else:
        df_final = pd.concat([df_final, df])
    controle_times += 1

# controle das colunas do df, apenas para análise durante desenvolvimento
colunas = df_final.columns

# seleção das colunas do df para carregamento no banco
df_carregamento = df_final[[
       'team.id', 'team.name','user.id', 'user.username', 'user.email']]

# renomeia as colunas
df_carregamento.rename(columns={'team.id' : 'time_id', 'team.name' : 'time_nome', 'user.id' : 'usuario_id', 'user.username' : 'usuario_nome', 'user.email' : 'usuario_email'}, inplace=True)

# cria o df para carregar no banco
df = df_carregamento.reset_index(drop=True)

# insere a data e hora na tabela para controle de atualizações
df['data_hora'] = data_hora

# conecta no banco
try:
    connection = psycopg2.connect(
        user= db_user,
        password= db_password,
        host= db_host,
        port= "5432",
        database= db_name
    )
    cursor = connection.cursor()

    # Testando a conexão
    cursor.execute("SELECT version();")
    record = cursor.fetchone()

except Exception as error:
    pass

# Fechando o cursor atual
cursor.close()
# Criando um novo cursor
cursor = connection.cursor()

# Executando a consulta TRUNCATE TABLE
truncate_query = f'TRUNCATE TABLE {db_schema}.{tabela};'
cursor.execute(truncate_query)

# Confirmando as alterações
connection.commit()

# Preparando a consulta SQL para inserção
insert_query = f'INSERT INTO {db_schema}.{tabela}  (time_id, time_nome, usuario_id, usuario_nome, usuario_email, data_hora) ' + 'VALUES (%s, %s, %s, %s, %s, %s);'

# Fechando o cursor atual
cursor.close()
# Criando um novo cursor
cursor = connection.cursor()

# Iterando sobre as linhas do DataFrame e inserindo no banco
for index, row in df.iterrows():
    cursor.execute(insert_query, (row['time_id'], row['time_nome'], row['usuario_id'], row['usuario_nome'], row['usuario_email'], row['data_hora']))

# Confirmando as alterações
connection.commit()

# Fechando a conexão
if connection:
            cursor.close()
            connection.close()