import requests
import yaml
import psycopg2
import pandas as pd
from pandas import json_normalize
from datetime import datetime
from Funcoes.consultaTabelas import tabela_existe
from Funcoes.criaTabelas import tabela_nova
from Config.pasta_local import pasta_local

# Abre o arquivo YAML com as configurações necessários. Validar o caminho que está salvo os arquivos para execução.
nome_arquivo_yaml = pasta_local+'\Config\config.yaml'

if __name__ == '__main__':
    with open(nome_arquivo_yaml,encoding='utf-8') as file:
        config = yaml.safe_load(file)

# Coleta os dados do arquivo YAML
token = config['token']
db_name = config['connector-oltp']['db_name']
db_user = config['connector-oltp']['db_user']
db_password = config['connector-oltp']['db_password']
db_host = config['connector-oltp']['db_host']
db_schema = config['db_schema']

# Informa a tabela que trabalharemos nesta etapa do ETL
tabela = 'clickup_folders'

# Comando SQL para criar a tabela
query = f'''
        CREATE TABLE {db_schema}.{tabela} (
            folder_id VARCHAR(20),
            folder_nome VARCHAR(200),
            folder_index VARCHAR(2),
            space_id VARCHAR(20),
            list_id VARCHAR(20),
            list_nome VARCHAR(200),
            list_index VARCHAR(5),
            list_total_tarefas VARCHAR(5),
            list_arquivada VARCHAR(10),
            data_hora TIMESTAMP
        );
        '''

# Verifica se já existe a tabela, caso não exista ela é criada
if tabela_existe(tabela,db_name,db_user,db_password,db_host):
      pass
else :
      tabela_nova(query,db_name,db_user,db_password,db_host)

# conecta no banco
try:
    connection = psycopg2.connect(
        user= db_user,
        password= db_password,
        host= db_host,
        port= "5432",
        database= db_name
    )
    cursor = connection.cursor()

    # Testando a conexão
    cursor.execute("SELECT version();")
    record = cursor.fetchone()

except Exception as error:
    pass

# Criando um novo cursor
cursor = connection.cursor()

# Consulta para selecionar os dados inseridos (ajuste conforme necessário)
select_query = f'SELECT DISTINCT space_id FROM {db_schema}.clickup_spaces;'

# Executando a consulta SELECT
cursor.execute(select_query)

# Obtendo e imprimindo os resultados
results = cursor.fetchall()

# Iterando sobre a resultado e criando uma lista
lista = [row[0] for row in results]

# função para invocar a consutla das workspaces dos times, trabalhará com loop caso tenha mais de um time
def requisicao(token_acesso, id):
    url = f'https://api.clickup.com/api/v2/space/{id}/folder'
    # print(url)

    # estrutura os headers conforme documentação utilizando o token salvo na pasta Config - arquivo config.yaml
    headers = {'Authorization': f'{token_acesso}'}

    # executa o request na url informando os headers
    response = requests.get(url, headers=headers)

    # coleta dos dados da respostsa
    data = response.json()
    try:
        pastas = data['folders']
    except:
        pass 
    return pastas

# Contador para controlar o append na lista
contador = 0

# loop das pastas na plataforma, desta maneira criamos o df com todos as pastas e listas nas pastas
for i in lista:
    lista_request = requisicao(token, i)
    # controle para saber se está na primeira "página".
    if contador == 0:
        lista_pastas = lista_request
    # caso não esteja na primeira página do while concatena os dois dfs.
    else:
        lista_pastas = lista_pastas + lista_request
    contador += 1

# tratamento dos dados para criar dataFrame
df_inicial = json_normalize(lista_pastas)
df_expandido = df_inicial.explode('lists')
df_listas = df_expandido['lists'].apply(pd.Series)
df_listas = df_listas[['id','name','orderindex','task_count','archived']]
df_listas.rename(columns={'id' : 'list_id','name' : 'list_nome','orderindex' : 'list_index','task_count' : 'list_total_tarefas','archived' : 'list_arquivada'}, inplace=True)


# Coleta data e hora para inserir no banco
data_hora = str(datetime.now())


# concatena por linha (join) entre a tabela inicial e colunas expandidas do status
df_final = pd.concat([df_expandido.drop(columns='lists'), df_listas], axis=1)

# renomeia as colunas do DataFrame
df_final.rename(columns={'id' : 'folder_id', 'name' : 'folder_nome', 'orderindex' : 'folder_index' , 'space.id' : 'space_id'}, inplace=True)

# insere data e hora no DataFrame
df_final['data_hora'] = data_hora

# seleciona as colunas para inserir no banco de dados
df_carregamento = df_final[['folder_id', 'folder_nome', 'folder_index', 'space_id', 'list_id', 'list_nome', 'list_index', 'list_total_tarefas', 'list_arquivada','data_hora']]

# Fechando o cursor atual
cursor.close()
# Criando um novo cursor
cursor = connection.cursor()

# Executa a consulta TRUNCATE TABLE
truncate_query = f'TRUNCATE TABLE {db_schema}.{tabela};'
cursor.execute(truncate_query)

# Confirma as alterações
connection.commit()

# Fecha o cursor atual
cursor.close()
# Cria um novo cursor
cursor = connection.cursor()

# Preparando a consulta SQL para inserção
insert_query = f'INSERT INTO {db_schema}.{tabela}  (folder_id , folder_nome, folder_index, space_id, list_id, list_nome, list_index, list_total_tarefas, list_arquivada, data_hora) ' + 'VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s);'

# Iterando sobre as linhas do DataFrame e inserindo no banco
for index, row in df_carregamento.iterrows():
    cursor.execute(insert_query, (row['folder_id'], row['folder_nome'], row['folder_index'], row['space_id'], row['list_id'], row['list_nome'], row['list_index'], row['list_total_tarefas'], row['list_arquivada'], row['data_hora']))

# Confirmando as alterações
connection.commit()

# Fechando o cursor atual
cursor.close()

if connection:
            cursor.close()
            connection.close()

