import requests
import yaml
import psycopg2
import pandas as pd
from pandas import json_normalize
from datetime import datetime
from Funcoes.consultaTabelas import tabela_existe
from Funcoes.criaTabelas import tabela_nova
from Config.pasta_local import pasta_local

# Coleta data e hora para inserir no banco
data_hora = datetime.now()

# Abre o arquivo YAML com as configurações necessários. Validar o caminho que está salvo os arquivos para execução.
nome_arquivo_yaml = pasta_local+'\Config\config.yaml'

if __name__ == '__main__':
    with open(nome_arquivo_yaml,encoding='utf-8') as file:
        config = yaml.safe_load(file)

# Coleta os dados do arquivo YAML
token = config['token']
db_name = config['connector-oltp']['db_name']
db_user = config['connector-oltp']['db_user']
db_password = config['connector-oltp']['db_password']
db_host = config['connector-oltp']['db_host']
db_schema = config['db_schema']

# Informa a tabela que trabalharemos nesta etapa do ETL
tabela = 'clickup_tasks'

# Comando SQL para criar a tabela
query = f'''
        CREATE TABLE {db_schema}.{tabela} (
task_id VARCHAR (20),
custom_id VARCHAR (20),
task_name VARCHAR (150),
description TEXT,
orderindex VARCHAR (100),
date_created VARCHAR (30),
date_updated VARCHAR (30),
date_closed VARCHAR (30),
date_done VARCHAR (30),
archived VARCHAR (10),
priority VARCHAR (20),
due_date VARCHAR (30),
start_date VARCHAR (30),
points VARCHAR (20),
team_id VARCHAR (15),
url VARCHAR (50),
permission_level VARCHAR (20),
status_status VARCHAR (50),
status_color VARCHAR (10),
status_type VARCHAR (20),
status_orderindex VARCHAR (5),
creator_id VARCHAR (20),
creator_username VARCHAR (100),
creator_color VARCHAR (10),
creator_email VARCHAR (50),
creator_profilePicture VARCHAR (10),
list_id VARCHAR (20),
list_name VARCHAR (150),
project_id VARCHAR (20),
project_name VARCHAR (50),
folder_id VARCHAR (20),
folder_name VARCHAR (50),
space_id VARCHAR (20),
priority_color VARCHAR (10),
priority_id VARCHAR (10),
priority_orderindex VARCHAR (10),
priority_priority VARCHAR (20),
Percentual_Honorários VARCHAR (20),
Percentual_Parceiro VARCHAR (20),
Anexos VARCHAR (20),
CNPJ VARCHAR (20),
Contador VARCHAR (20),
Contato_do_Cliente VARCHAR (20),
Contatos TEXT,
Contatos_Do_Contador TEXT,
Email_do_Cliente TEXT,
Email_do_responsável TEXT,
Email_do_Contato TEXT,
Parceiro VARCHAR (50),
Setor VARCHAR (50),
Valor_Compensado VARCHAR (50),
Valor_Indeferido VARCHAR (50),
Valor_Pago VARCHAR (50),
Valor_do_Crédito_Pedido VARCHAR (50),
Valor_do_crédito_apurado VARCHAR (50),
assignees_id VARCHAR (20),
assignees_username VARCHAR (50),
id_checklist VARCHAR (50),
name_checklist VARCHAR (200),
date_created_checklist VARCHAR (30),
orderindex_checklist VARCHAR (10),
creator_checklist VARCHAR (20),
resolved_checklist VARCHAR (10),
unresolved_checklist VARCHAR (10),
id_item_checklist VARCHAR (50),
name_item_checklist TEXT,
date_created_item_checklist VARCHAR (30),
orderindex_item_checklist VARCHAR (10),
assignee_item_checklist VARCHAR (20),
resolved_item_checklist VARCHAR (20),
parent_item_checklist VARCHAR (20),
name_tag VARCHAR (20),
tag_fg VARCHAR (10),
tag_bg VARCHAR (10),
creator_tag VARCHAR (20),
percent_complete_progress VARCHAR (10),
link_id_linked_tasks VARCHAR (20),
date_created_linked_tasks VARCHAR (30),
userid_linked_tasks VARCHAR (20),
workspace_id_linked_tasks VARCHAR (20),
data_hora VARCHAR (30)
        );
        '''

# Verifica se já existe a tabela, caso não exista ela é criada
if tabela_existe(tabela,db_name,db_user,db_password,db_host):
    #   print(f'Tabela "{tabela}" já existe!')
      pass
else :
      tabela_nova(query,db_name,db_user,db_password,db_host)

# conecta no banco
try:
    connection = psycopg2.connect(
        user= db_user,
        password= db_password,
        host= db_host,
        port= "5432",
        database= db_name
    )
    cursor = connection.cursor()
    
    # Testando a conexão
    cursor.execute("SELECT version();")
    record = cursor.fetchone()


except Exception as error:
    pass

# Criando um novo cursor
cursor = connection.cursor()

# Consulta para selecionar os dados inseridos (ajuste conforme necessário)
select_query = f'SELECT DISTINCT list_id FROM {db_schema}.clickup_folders;'

# Executando a consulta SELECT
cursor.execute(select_query)

# Obtendo e imprimindo os resultados
results = cursor.fetchall()

# Iterando sobre a resultado e criando uma lista
lista = [row[0] for row in results]

# Remove ids com valor "NaN"
lista.remove('NaN')

# =============================================================================== #


# função para invocar a consutla das workspaces dos times, trabalhará com loop caso tenha mais de um time
def requisicao(token_acesso, id, pagina):
    url = f'https://api.clickup.com/api/v2/list/{id}/task?page={pagina}'
    # print(url)

    # estrutura os headers conforme documentação utilizando o token salvo na pasta Config - arquivo config.yaml
    headers = {'Authorization': f'{token_acesso}'}

    # executa o request na url informando os headers
    response = requests.get(url, headers=headers)

    # coleta dos dados da respostsa
    data = response.json()
    try:
        tarefas = data
    except:
        pass 
    return tarefas

# Contador para controlar o append na lista
contador = 0

# para validar tamanho da lista
tamanho_lista = len(lista)
# loop das pastas na plataforma, desta maneira criamos o df com todos as pastas e listas nas pastas
for i in lista:
    ultima_pagina = False
    pagina_atual = 0
    while ultima_pagina  != True:
        lista_request = requisicao(token, i, pagina_atual)
        # controle para saber se está na primeira "página".
        if contador == 0:
            lista_tarefas = lista_request['tasks']
            ultima_pagina = lista_request['last_page']
            pagina_atual += 1
        # caso não esteja na primeira página do while concatena os dois dfs.
        else:
            lista_tarefas = lista_tarefas + lista_request['tasks']
            ultima_pagina = lista_request['last_page']
            pagina_atual += 1
        contador += 1
    
    
# tratamento dos dados para criar dataFrame
df = json_normalize(lista_tarefas)
df = df.rename(columns={'id': 'task_id'})
df_campos_personalizados = df[['task_id' , 'custom_fields']]
df_explode = df_campos_personalizados.explode('custom_fields')
df = df.drop(columns='custom_fields')
df_expandido = df_explode['custom_fields'].apply(pd.Series)
df_transitorio = pd.concat([df_campos_personalizados.drop(columns='custom_fields'), df_expandido], axis=1)
df_expandido = df_expandido[['id','name','type','type_config','date_created', 'hide_from_guests','required','value']]
df_pivot = df_transitorio.pivot_table(index='task_id',columns='name', values='value', aggfunc='sum')
df_reset_index = df_pivot.reset_index()
df_join = pd.merge(df, df_pivot, on='task_id', how='left')

# seleciona as colunas para continuar a expansão das colunas
df_colunas_selecionadas = df_join[['task_id','custom_id', 'name', 'description','orderindex','date_created','date_updated','date_closed','date_done','archived','assignees','watchers','checklists','tags','parent','priority','due_date','start_date','points','linked_tasks','team_id','url','permission_level','status.status','status.color','status.type','status.orderindex','creator.id','creator.username','creator.color','creator.email','creator.profilePicture','list.id','list.name','project.id','project.name','folder.id','folder.name','space.id','priority.color','priority.id','priority.orderindex','priority.priority','% Honorários','% Parceiro','Anexos','CNPJ','Contador','Contato do Cliente','Contatos','Contatos Do Contador','E-mail do Cliente','E-mail do responsável','Email do Contato','Parceiro','Setor','Valor Compensado','Valor Indeferido','Valor Pago','Valor do Crédito Pedido','Valor do crédito apurado','📈 Progress']]

# renomeia as colunas
df_carregamento = df_colunas_selecionadas.rename(columns={'name' : 'task_name',
'status.status' : 'status_status',
'status.color' : 'status_color',
'status.type' : 'status_type',
'status.orderindex' : 'status_orderindex',
'creator.id' : 'creator_id',
'creator.username' : 'creator_username',
'creator.color' : 'creator_color',
'creator.email' : 'creator_email',
'creator.profilePicture' : 'creator_profilePicture',
'sharing.public' : 'sharing_public',
'sharing.public_share_expires_on' : 'sharing_public_share_expires_on',
'sharing.public_fields' : 'sharing_public_fields',
'sharing.token' : 'sharing_token',
'sharing.seo_optimized' : 'sharing_seo_optimized',
'list.id' : 'list_id',
'list.name' : 'list_name',
'list.access' : 'list_access',
'project.id' : 'project_id',
'project.name' : 'project_name',
'project.hidden' : 'project_hidden',
'project.access' : 'project_access',
'folder.id' : 'folder_id',
'folder.name' : 'folder_name',
'folder.hidden' : 'folder_hidden',
'folder.access' : 'folder_access',
'space.id' : 'space_id',
'priority.color' : 'priority_color',
'priority.id' : 'priority_id',
'priority.orderindex' : 'priority_orderindex',
'priority.priority' : 'priority_priority',
'% Honorários' : 'Percentual_Honorários',
'% Honorários Tributo Devido' : 'Percentual_Honorários_Tributo_Devido',
'% Parceiro' : 'Percentual_Parceiro',
'% dos Honorários totais' : 'Percentual_dos_Honorários_totais',
'CNAE principal' : 'CNAE_principal',
'Client Assets' : 'Client_Assets',
'Contato do Cliente' : 'Contato_do_Cliente',
'Contatos Do Contador' : 'Contatos_Do_Contador',
'Crédito x Faturamento bruto' : 'Crédito_x_Faturamento_bruto',
'Crédito x PIS COFINS original' : 'Crédito_x_PIS_COFINS_original',
'Custom Dropdown Opcao Tributaria' : 'Custom_Dropdown_Opcao_Tributaria',
'Custom Number CNPJ' : 'Custom_Number_CNPJ',
'Custom Text Consultor' : 'Custom_Text_Consultor',
'Custom Text Conta Bancaria' : 'Custom_Text_Conta_Bancaria',
'Custom Text Contador' : 'Custom_Text_Contador',
'Custom Text Contatos' : 'Custom_Text_Contatos',
'Custom Text Contatos Do Contador' : 'Custom_Text_Contatos_Do_Contador',
'Custom Text N Do Processo' : 'Custom_Text_N_Do_Processo',
'Custom Text Nome Do Contato' : 'Custom_Text_Nome_Do_Contato',
'Dados Bancarios' : 'Dados_Bancarios',
'Data da petição' : 'Data_da_petição',
'Data do deferimento da habilitação' : 'Data_do_deferimento_da_habilitação',
'Data do pedido de habilitação' : 'Data_do_pedido_de_habilitação',
'Data do trânsito' : 'Data_do_trânsito',
'Data final do cálculo' : 'Data_final_do_cálculo',
'Data inicial do cálculo' : 'Data_inicial_do_cálculo',
'Dossiê Habilitação' : 'Dossiê_Habilitação',
'E-mail do Cliente' : 'Email_do_Cliente',
'E-mail do responsável' : 'Email_do_responsável',
'Email do Contato' : 'Email_do_Contato',
'Faturamento do período' : 'Faturamento_do_período',
'Honorários Previstos TD' : 'Honorários_Previstos_TD',
'Honorários Tributo Devido' : 'Honorários_Tributo_Devido',
'Honorários recebidos TD' : 'Honorários_recebidos_TD',
'Honorários totais' : 'Honorários_totais',
'ICMS - ST' : 'ICMS_ST',
'Número da Ação' : 'Número_da_Ação',
'Número do Processo' : 'Número_do_Processo',
'Número do processo administrativo' : 'Número_do_processo_administrativo',
'PIS/COFINS do período' : 'PIS_COFINS_do_período',
'Parte ré' : 'Parte_ré',
'Pasta SharePoint' : 'Pasta_SharePoint',
'Pasta de Contratos' : 'Pasta_de_Contratos',
'Prazo Previsto para Recebimento' : 'Prazo_Previsto_para_Recebimento',
'Progresso da tarefa' : 'Progresso_da_tarefa',
'Qual foi o Motivo para o não fechamento do contrato? Detalhe aqui o motivo pelo qual o cliente não f' : 'Qual_foi_o_Motivo_para_o_não_fechamento_do_contrato_Detalhe_aqui_o_motivo_pelo_qual_o_cliente_não_f',
'Regime tributário' : 'Regime_tributário',
'Revisão TD' : 'Revisão_TD',
'Selecione a situação atual do cliente: ' : 'Selecione_a_situação_atual_do_cliente',
'Telefone responsável' : 'Telefone_responsável',
'Tempo de trâmite judicial' : 'Tempo_de_trâmite_judicial',
'Tempo trâmite administrativo' : 'Tempo_trâmite_administrativo',
'Valor Compensado' : 'Valor_Compensado',
'Valor Faturado' : 'Valor_Faturado',
'Valor Indeferido' : 'Valor_Indeferido',
'Valor Pago' : 'Valor_Pago',
'Valor do Crédito Compensado' : 'Valor_do_Crédito_Compensado',
'Valor do Crédito Pedido' : 'Valor_do_Crédito_Pedido',
'Valor do Crédito Ressarcido' : 'Valor_do_Crédito_Ressarcido',
'Valor do Saldo do Crédito' : 'Valor_do_Saldo_do_Crédito',
'Valor do crédito apurado' : 'Valor_do_crédito_apurado',
'Órgão julgador' : 'Órgão_julgador',
'📈 Progress' : 'Progress'
})

################################# assignees #################################
df_assignees = df_carregamento.explode('assignees').reset_index(drop=True)

assignees = df_assignees[['task_id','assignees']]

assignees_final = json_normalize(assignees['assignees'])

assignees_final = assignees_final[['id', 'username']]

assignees_final = assignees_final.rename(columns={'id' : 'assignees_id', 'username' : 'assignees_username'})

df_carregamento_final = pd.concat([df_assignees.drop(columns='assignees'), assignees_final], axis=1)
################################# assgnees #################################

################################# checklists #################################
df_checklists = df_carregamento_final.explode('checklists').reset_index(drop=True)

checklists = df_checklists[['task_id','checklists']]

checklists_final = json_normalize(checklists['checklists'])

checklists_final = checklists_final[['id', 'name', 'date_created', 'orderindex', 'creator', 'resolved', 'unresolved', 'items']]

checklists_final = checklists_final.rename(columns={'id' : 'id_checklist', 'name' : 'name_checklist', 'date_created' : 'date_created_checklist', 'orderindex' : 'orderindex_checklist', 'creator' : 'creator_checklist', 'resolved' : 'resolved_checklist', 'unresolved' : 'unresolved_checklist', 'items' : 'items_checklist'})

df_carregamento_final = pd.concat([df_checklists.drop(columns=['checklists']), checklists_final], axis=1)

df_items_checklist = df_carregamento_final.explode('items_checklist').reset_index(drop=True)

checklists_items = json_normalize(df_items_checklist['items_checklist'])

checklists_items = checklists_items[['id', 'name', 'date_created', 'orderindex', 'assignee', 'resolved', 'parent', 'children']]

checklists_items = checklists_items.rename(columns={'id' : 'id_item_checklist', 'name' : 'name_item_checklist', 'date_created' : 'date_created_item_checklist', 'orderindex' : 'orderindex_item_checklist', 'assignee' : 'assignee_item_checklist', 'resolved' : 'resolved_item_checklist', 'parent' : 'parent_item_checklist', 'children' : 'children_item_checklist'})

df_carregamento_final = pd.concat([df_items_checklist.drop(columns=['items_checklist']), checklists_items], axis=1)
################################# checklists #################################

################################# tags #################################
df_tags = df_carregamento_final.explode('tags').reset_index(drop=True)

tags = df_tags[['task_id','tags']]

tags_final = json_normalize(tags['tags'])

tags_final = tags_final.rename(columns={'name' : 'name_tag', 'creator' : 'creator_tag'})

df_carregamento_final = pd.concat([df_tags.drop(columns=['tags']), tags_final], axis=1)
################################# tags #################################

################################# Progress #################################
df_progress = json_normalize(df_carregamento_final['Progress'])

df_progress = df_progress.rename(columns={'percent_complete' : 'percent_complete_progress'})

df_carregamento_final = pd.concat([df_carregamento_final.drop(columns=['Progress']), df_progress], axis=1)
################################# Progress #################################

################################# linked_tasks #################################
df_linked_tasks = df_carregamento_final.explode('linked_tasks').reset_index(drop=True)

linked_tasks = df_linked_tasks[['task_id','linked_tasks']]

linked_tasks = json_normalize(linked_tasks['linked_tasks'])

linked_tasks = linked_tasks[['link_id', 'date_created', 'userid', 'workspace_id']]

linked_tasks_final = linked_tasks.rename(columns={'link_id' : 'link_id_linked_tasks', 'date_created' : 'date_created_linked_tasks', 'userid' : 'userid_linked_tasks', 'workspace_id' : 'workspace_id_linked_tasks'})

df_carregamento_final = pd.concat([df_linked_tasks.drop(columns=['linked_tasks']), linked_tasks_final], axis=1)
################################# linked_tasks #################################

################################# Parceiro #################################
df_parceiros = df_carregamento_final.explode('Parceiro')

################################# Parceiro #################################

################################# Setor #################################
df_setor = df_parceiros.explode('Setor')

################################# Setor #################################

# insere data e hora no dataFrame
df_setor['data_hora'] = data_hora

# confirma as colunas para carregar no banco.
df_carregamento_ultima_etapa = df_setor[['task_id',	'custom_id','task_name','description','orderindex','date_created',	'date_updated',	'date_closed',	'date_done',	'archived',	'priority',	'due_date',	'start_date',	'points',	'team_id',	'url',	'permission_level',	'status_status',	'status_color',	'status_type',	'status_orderindex',	'creator_id',	'creator_username',	'creator_color',	'creator_email',	'creator_profilePicture',	'list_id',	'list_name',	'project_id',	'project_name',	'folder_id',	'folder_name',	'space_id',	'priority_color',	'priority_id',	'priority_orderindex',	'priority_priority',	'Percentual_Honorários',	'Percentual_Parceiro',	'Anexos',	'CNPJ',	'Contador',	'Contato_do_Cliente',	'Contatos',	'Contatos_Do_Contador',	'Email_do_Cliente',	'Email_do_responsável',	'Email_do_Contato',	'Parceiro',	'Setor',	'Valor_Compensado',	'Valor_Indeferido',	'Valor_Pago',	'Valor_do_Crédito_Pedido',	'Valor_do_crédito_apurado',	'assignees_id',	'assignees_username',	'id_checklist',	'name_checklist',	'date_created_checklist',	'orderindex_checklist',	'creator_checklist',	'resolved_checklist',	'unresolved_checklist',	'id_item_checklist',	'name_item_checklist',	'date_created_item_checklist',	'orderindex_item_checklist',	'assignee_item_checklist',	'resolved_item_checklist',	'parent_item_checklist',	'name_tag',	'tag_fg',	'tag_bg',	'creator_tag',	'percent_complete_progress',	'link_id_linked_tasks',	'date_created_linked_tasks',	'userid_linked_tasks',	'workspace_id_linked_tasks',	'data_hora']]


# Fecha o cursor atual
cursor.close()
# Cria um novo cursor
cursor = connection.cursor()

# Comando SQL para criar a tabela
insert_query = f'INSERT INTO {db_schema}.{tabela}  (task_id, custom_id,task_name,description,orderindex,date_created, date_updated, date_closed, date_done, archived, priority, due_date, start_date, points, team_id, url, permission_level, status_status, status_color, status_type, status_orderindex, creator_id, creator_username, creator_color, creator_email, creator_profilePicture, list_id, list_name, project_id, project_name, folder_id, folder_name, space_id, priority_color, priority_id, priority_orderindex, priority_priority, Percentual_Honorários, Percentual_Parceiro, Anexos, CNPJ, Contador, Contato_do_Cliente, Contatos, Contatos_Do_Contador, Email_do_Cliente, Email_do_responsável, Email_do_Contato, Parceiro, Setor, Valor_Compensado, Valor_Indeferido, Valor_Pago, Valor_do_Crédito_Pedido, Valor_do_crédito_apurado, assignees_id, assignees_username, id_checklist, name_checklist, date_created_checklist, orderindex_checklist, creator_checklist, resolved_checklist, unresolved_checklist, id_item_checklist, name_item_checklist, date_created_item_checklist, orderindex_item_checklist, assignee_item_checklist, resolved_item_checklist, parent_item_checklist, name_tag, tag_fg, tag_bg, creator_tag, percent_complete_progress, link_id_linked_tasks, date_created_linked_tasks, userid_linked_tasks, workspace_id_linked_tasks, data_hora) ' + 'VALUES (%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s);'

# Executa a consulta TRUNCATE TABLE
truncate_query = f'TRUNCATE TABLE {db_schema}.{tabela};'
cursor.execute(truncate_query)
connection.commit()

# Iterando sobre a resultado e criando uma lista
for index, row in df_carregamento_ultima_etapa.iterrows():
    try:
        cursor.execute(insert_query, (row['task_id'],	row['custom_id'],	row['task_name'],	row['description'],	row['orderindex'],	row['date_created'],	row['date_updated'],	row['date_closed'],	row['date_done'],	row['archived'],	row['priority'],	row['due_date'],	row['start_date'],	row['points'],	row['team_id'],	row['url'],	row['permission_level'],	row['status_status'],	row['status_color'],	row['status_type'],	row['status_orderindex'],	row['creator_id'],	row['creator_username'],	row['creator_color'],	row['creator_email'],	row['creator_profilePicture'],	row['list_id'],	row['list_name'],	row['project_id'],	row['project_name'],	row['folder_id'],	row['folder_name'],	row['space_id'],	row['priority_color'],	row['priority_id'],	row['priority_orderindex'],	row['priority_priority'],	row['Percentual_Honorários'],	row['Percentual_Parceiro'],	row['Anexos'],	row['CNPJ'],	row['Contador'],	row['Contato_do_Cliente'],	row['Contatos'],	row['Contatos_Do_Contador'],	row['Email_do_Cliente'],	row['Email_do_responsável'],	row['Email_do_Contato'],	row['Parceiro'],	row['Setor'],	row['Valor_Compensado'],	row['Valor_Indeferido'],	row['Valor_Pago'],	row['Valor_do_Crédito_Pedido'],	row['Valor_do_crédito_apurado'],	row['assignees_id'],	row['assignees_username'],	row['id_checklist'],	row['name_checklist'],	row['date_created_checklist'],	row['orderindex_checklist'],	row['creator_checklist'],	row['resolved_checklist'],	row['unresolved_checklist'],	row['id_item_checklist'],	row['name_item_checklist'],	row['date_created_item_checklist'],	row['orderindex_item_checklist'],	row['assignee_item_checklist'],	row['resolved_item_checklist'],	row['parent_item_checklist'],	row['name_tag'],	row['tag_fg'],	row['tag_bg'],	row['creator_tag'],	row['percent_complete_progress'],	row['link_id_linked_tasks'],	row['date_created_linked_tasks'],	row['userid_linked_tasks'],	row['workspace_id_linked_tasks'],	row['data_hora']))
        connection.commit()
    except Exception as e:
        connection.rollback()
 
# Confirmando as alterações
connection.commit()

# Fechando o cursor atual e conexão
if connection:
            cursor.close()
            connection.close()